-- Love3D : Perspective Demo 8 --

function love.load()
	monitorWidth = 1024
	monitorHeight = 768

	numPoints = 8
	points = {}
	
	for i=1, numPoints do
		point = {}
		point.x = 0 -- Please note, ball.x/ball.y is the position in what WE see
		point.y = 0 -- ball.xPos/ball.yPos/ball.zPos is the position in 3D space
		if i == 1 then
			point.xPos = -100
			point.yPos = -100
			point.zPos = -100
		elseif i == 2 then
			point.xPos = 100
			point.yPos = -100
			point.zPos = -100
		elseif i == 3 then
			point.xPos = 100
			point.yPos = 100
			point.zPos = -100
		elseif i == 4 then
			point.xPos = -100
			point.yPos = 100
			point.zPos = -100
		end
		
		if i == 5 then
			point.xPos = -100
			point.yPos = -100
			point.zPos = 100
		elseif i == 6 then
			point.xPos = 100
			point.yPos = -100
			point.zPos = 100
		elseif i == 7 then
			point.xPos = 100
			point.yPos = 100
			point.zPos = 100
		elseif i == 8 then
			point.xPos = -100
			point.yPos = 100
			point.zPos = 100
		end
		point.scale = 1 -- This is an important value for perspective (obviously)
		table.insert(points, point)
	end
	faces = {
	{1,2,3,4, color={255,0,0}},
	{5,6,7,8, color={0,255,0}},
	{1,4,8,5, color={0,0,255}},
	{1,2,6,5, color={255,255,0}},
	{2,3,7,6, color={255,0,255}},
	{3,4,8,7, color={0,255,255}}
	}
	
	vpX = monitorWidth/2 --  These are the vanishing points, 
	vpY = monitorHeight/2 -- in this case, it is the center of the screen.
	fl = 250 -- This number is usually 200 - 300, to simulate perspective.
end

function love.update(dt)
	local angleX = (love.mouse.getX() - vpX) * (0.05 * dt)
	local angleY = (love.mouse.getY() - vpY) * (0.05 * dt)
	for i, v in ipairs(points) do
		if v.zPos > fl * -1 then
			perspective2(v, true, v.xPos, v.yPos, v.zPos)
			rotateX(v, angleX)
			rotateY(v, angleY)
		end
	end
	
	for i, v in ipairs(faces) do
	
		v.averageZ = (points[v[1]].zPos+points[v[2]].zPos+points[v[3]].zPos+points[v[4]].zPos)/4
		
	end
	
	table.sort(faces, function(a, b) return a.averageZ > b.averageZ end)
	
end

function love.draw()
	for i, v in ipairs(points) do
		love.graphics.circle("fill", v.x, v.y, 10 * v.scale, 25)
		--love.graphics.setLine(2, "smooth")
	end
	love.graphics.setColor(255,0,0)
	love.graphics.point(512,384)
	for i, v in ipairs(faces) do
		love.graphics.setColor(v.color)
		local point1, point2, point3, point4 = points[v[1]], points[v[2]], points[v[3]], points[v[4]]
		love.graphics.polygon('fill', point1.x, point1.y, point2.x, point2.y, point3.x, point3.y, point4.x, point4.y)
	end
	
	love.graphics.setColor(255,255,255)
end

function perspective(object, isScale)
	local scale = fl / (fl + object.zPos)
	if isScale then
		object.scale = scale
	end
	object.x = vpX + object.xPos * scale
	object.y = vpY + object.yPos * scale
end

function perspective2(object, isScale, xPos, yPos, zPos)
	local scale = fl / (fl + zPos)
	if isScale then
		object.scale = scale
	end
	object.x = vpX + xPos * scale
	object.y = vpY + yPos * scale
end

function rotateX(v, angleX)
	local cosX = math.cos(angleX)
	local sinX = math.sin(angleX)
	local y1 = v.yPos * cosX - v.zPos * sinX
	local z1 = v.zPos * cosX + v.yPos * sinX
	v.yPos = y1
	v.zPos = z1
end

function rotateY(v, angleY)
	local cosY = math.cos(angleY)
	local sinY = math.sin(angleY)
	local x1 = v.xPos * cosY - v.zPos * sinY
	local z1 = v.zPos * cosY + v.xPos * sinY
	v.xPos = x1
	v.zPos = z1
end